
/***********************************************************************************/
/* Copyright (c) 2017, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_xml_seqnum                                           */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       assign sequence number to xml tags                               */
/* PARAMETERS:    exclude_dsn: name of table containing numbers that should        */
/*                        not be used for sequence number.                         */ 
/*                exclude_var: variable in exclude_dsn that contains the seq number*/
/*                             that should be excluded                             */  
/*                infname: file reference of input xml                             */
/*                outfname: file reference of output xml                           */
/*                linecntVar: name of macro variable to return total number of     */
/*                            in the xml.                                          */
/* NOTE:          replacing '.' in 'SeqNum="."' with unique seq number.            */
/*                although FinCEN CTRX user's guide requires the sequence number be*/
/*                consecutive and starts with 1, the efile got accepted as long    */
/*                as the sequence numbers are unique.                              */
/***********************************************************************************/


%macro ecmrr_efile_xml_seqnum(exclude_dsn=,
                              exclude_var=, 
                              infname=, 
                              outfname=,
                              linecntVar=line_cnt);

%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;


%let exclude_cnt=0;
%if %str(&exclude_dsn) ne %str() %then
%do; 
     %let exclude_list=;
     proc sql noprint;
         select distinct strip(put(&exclude_var,best.)) 
                into :exclude_list separated by ' ' 
          from &exclude_dsn
          ;
		  %let exclude_cnt=&sqlobs;
    quit;
%end;

data _null_;
    retain offset 0;
    infile &infname end=eof ;
    file &outfname;
    input;	

    if index(_infile_,'SeqNum="."') then
    do; 
    %if &exclude_cnt > 0 %then
    %do;
           /* The line number plus offset is used for sequence number  */
	   /* If the current seqNum cannot be use, add one to offset   */
	   do while (_n_ + offset in (&exclude_list));
	      offset + 1;
	   end;
	   seqNum = _n_  + offset;
    %end;
    %else 
    %do;
           seqNum=_n_;
    %end;
       _infile_=tranwrd(_infile_,'SeqNum="."','SeqNum="'||strip(put(seqNum,best.))||'"');
    end;
    put _infile_;
	if eof then call symput("&lineCntVar",strip(put(_n_,best.)));
run;

%if &syserr > 4 %then
%do;
    %let successYN=N;	 
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
    %goto mExit;
%end;

%mexit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;


 
